/*

  Programa con Arduino y tarjeta ESP32 Ver 2 que usa bluetooth para recibir los códigos que 
  envía el ESP32_DecoderTx vía Bluetooth, el cual ha recibido códigos IR desde el Control
  Remoto del Decodificador de TV.

  En este programa se traducen los datos recibidos para convertirse nuevamente en el formato:
  1.- 64 64 1 254
  7.- 64 64 7 248

  y generar una portadora de 38 KHz con un NE555 al cual se le inyectan por el PIN RESET los
  códigos. La salida OUT del 555 se lleva a un LED Infrarrojo que apunta al Decoder TV, con lo
  cual se logra el cambio de canal, ajustar volumen, etc., en la TV.

  Alfredo Segura, Querétaro, México febrero de 2022.
*/

#include "BluetoothSerial.h"
#include "IRsend.h"

int IROut = 32;     // Salida, Código IR para enviar al NE555
int ledPin = 25;    // LED de conexión y actividad

int edoLed = LOW;
int edoAntLed;
int edoActLed;
bool conectado;
int i;
int bandera = 0;
String comando;
uint32_t hexcomando;

volatile int interruptCounter;
int totalInterruptCounter;

IRsend irtrans(IROut);

hw_timer_t * timer = NULL;
portMUX_TYPE timerMux = portMUX_INITIALIZER_UNLOCKED;

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

BluetoothSerial SerialBT;

void IRAM_ATTR onTimer() {
  portENTER_CRITICAL_ISR(&timerMux);
  interruptCounter++;
  portEXIT_CRITICAL_ISR(&timerMux);
}

void setup() {
  timer = timerBegin(0, 80, true);
  timerAttachInterrupt(timer, &onTimer, true);
  timerAlarmWrite(timer, 12000000, true);
  timerAlarmEnable(timer);

  pinMode(IROut, OUTPUT);
  digitalWrite(IROut, LOW);               // IROut, pin 32 en LOW al iniciar
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, LOW);
  
  Serial.begin(115200);
  SerialBT.begin("ESP32_DecoderTVRx");      //El nombre de este receptor Bluetooth (Esclavo = Server)
  Serial.println("ESP32_DecoderTVRx esta listo para recibir datos desde el ESP32_DecoderTVTx");
}

void loop() {

  if (interruptCounter > 0) {                 // Parpadea ledPin para actividad
    edoAntLed = edoActLed;
    edoActLed = digitalRead(ledPin);
    edoLed = !edoLed;
    digitalWrite(ledPin, edoLed);
    interruptCounter = 0;
  }
  i = 0;
  // Atención a los comandos vía Bluetooth
  if (SerialBT.available()) {         // Los datos recibidos corresponden exclusivamente al comdc
    comando = SerialBT.read();        // es decir el valor del comando complementado
    Serial.println(comando);
  /*
   * Debido a que ya se conoce la estructura de la ráfaga IR para este Control remoto (64 64 5 95) para 
   * el caso de la Tecla 5, se mandará a ejecutar el IR-Tx enviando 2 veces 64, el comando vuelto a complementar
   * y el comando recibido que ya está complementado.
   */
    switch (comando.toInt()) {
      case 127:     // 1
        hexcomando = 0x0202807F;
        break;
      case 191:     // 2
        hexcomando = 0x020240BF;
        break;
      case 63:     // 3
        hexcomando = 0x0202C03F;
        break;
      case 223:     // 4
        hexcomando = 0x020220DF;
        break;
      case 95:     // 5
        hexcomando = 0x0202A05F;
        break;
      case 159:     // 6
        hexcomando = 0x0202609F;
        break;
      case 31:     // 7
        hexcomando = 0x0202E01F;
        break;
      case 239:     // 8
        hexcomando = 0x020210EF;
        break;
      case 255:     // 0
        hexcomando = 0x020200FF;
        break;
      case 87:     // VOL +
        hexcomando = 0x0202A857;
        break;
      case 199:     // VOL -
        hexcomando = 0x020238C7;
        break;
      case 7:     // CH +
        hexcomando = 0x0202F807;
        break;
      case 135:     // CH -
        hexcomando = 0x02027887;
        break;
      case 207:     // MUTE
        hexcomando = 0x020230CF;
        break;
      case 189:     // INFO
        hexcomando = 0x020242BD;
        break;
      case 143:     // DWN
        hexcomando = 0x0202708F;
        break;
      case 125:     // EXIT
        hexcomando = 0x0202827D;
        break;
      case 247:     // IZQ
        hexcomando = 0x020208F7;
        break;
      case 79:     // OK
        hexcomando = 0x0202B04F;
        break;
      case 119:     // DER
        hexcomando = 0x02028877;
        break;
      case 253:     // MENU
        hexcomando = 0x020202FD;
        break;
      case 47:     // UP
        hexcomando = 0x0202D02F;
        break;
      case 167:     // GUIA
        hexcomando = 0x020258A7;
        break;
      case 55:     // AUDIO
        hexcomando = 0x0202C837;
        break;
      case 215:     // SUBT
        hexcomando = 0x020228D7;
        break;
      case 39:     // IDIOMA
        hexcomando = 0x0202D827;
        break;
      case 61:     // BACK
        hexcomando = 0x0202C23D;
        break;
    }   // Termina Switch
    irtrans.sendNEC(hexcomando);
  } 
  delay(200);
}
