/*
Este programa habilita un ESP32 como emisor (Master = Cliente) via Bluetooth de los
códigos IR que recibe por medio de un sensor desde el Control Remoto del Decoder TV
desde otra ubicación alejada de dicho decoder. Por medio del Bluetooth (BT) transfiere
los códigos a otro ESP32_DecoderRx, el cual reproducirá a través de un generador de
38 KHz y un LED emisor IR los mismo códigos, para apuntar directamente al DecoderTV
y con ésto se logrará el Control Remoto lejano o con obstáculos.

Se probaron los códigos que genera el Control Remoto con el decodificador y se 
encontraron los siguientes valores, todos con el formato 64 64 1 254. Los primeros 2
números son siempre '64, el 3ero es el código y el 4to es el resultado de 255 - código:

1.- 64 64 1 254   2.- 64 64 2 253  3.- 64 64 3 252  4.- 64 64 4 251   5.- 64 64 5 250
6.- 64 64 6 249   7.- 64 64 7 248  8.- 64 64 8 247  9.- 64 64 9 246   0.- 64 64 0 255

VOL+.- 64 64 21 234     VOL-.- 64 64 28 227     MUSIC.- 64 64 29 226
MUTE.- 64 64 12 243     INFO.- 64 64 66 189     EXIT.- 64 64 65 190

MENU.- 64 64 64 191     GUIA.- 64 64 26 229     TEMP.- 64 64 18 237
BACK.- 64 64 67 188     VOD.- 64 64 71 184      VOUT.- 64 64 68 187

BOX.- 64 64 24 231      AUDIO.- 64 64 24 236    SUBT.- 64 64 20 235
IDIOMA.- 64 64 27 228   UP.- 64 64 11 244       DWN.- 64 64 14 241
IZQ.- 64 64 16 239      DER.- 64 64 17 238

Tm5 = 277, T2=1108 (duración de los pulsos de la señal IR).

Alfredo Segura, Mexico febrero de 2022
*/

#include "BluetoothSerial.h"
#include "IRrecv.h"

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

int ledPin = 25;    // LED de actividad y conexión exitosa
int IR = 34;        // Entrada de la señal IR del VR38
int uno = 32;       // El pin 32 va  High a un LED Rojo
int dos = 33;       // El pin 33 va  High a un LED Azul. Estos 2 LEDs se encienden y apagan
                    // simultaneos ante un comando IR válido.

int edoLed = LOW;
int edoAntLed;
int edoActLed;
bool conectado;
int i = 0;
int j = 0;
int m = 0;
int adr = 0;
int adrc = 0;
int cmd = 0;
int cmdc = 0;
uint32_t comando;
int bandera = 0;

volatile int interruptCounter;
int totalInterruptCounter;

IRrecv irrecv(IR);
decode_results results;

hw_timer_t * timer = NULL;
portMUX_TYPE timerMux = portMUX_INITIALIZER_UNLOCKED;

BluetoothSerial SerialBT;

void IRAM_ATTR onTimer() {
  portENTER_CRITICAL_ISR(&timerMux);
  interruptCounter++;
  portEXIT_CRITICAL_ISR(&timerMux);
}

void setup() {
  timer = timerBegin(0, 80, true);
  timerAttachInterrupt(timer, &onTimer, true);
  timerAlarmWrite(timer, 12000000, true);
  timerAlarmEnable(timer);

  irrecv.enableIRIn();
  pinMode(ledPin, OUTPUT);
  pinMode(uno, OUTPUT);
  pinMode(dos, OUTPUT);
  
  Serial.begin(115200);
  SerialBT.begin("ESP32_DecoderTVTx", true);    // El nombre de este dispositivo transmisor Master = Cliente
  Serial.println("El transmisor está en modo Master, conectandose con el ESP32_DecoderTVRx...");
  conectado = SerialBT.connect("ESP32_DecoderTVRx");

  if (conectado) {
    Serial.println("Conectado exitosamente...");
  }else {
     while (!SerialBT.connected(10000)) {
       Serial.println("No se pudo conectar. El ESP32_DecoderTVRx debe estar encendido y cercano");
       Serial.println("Reinicie la aplicación...");
      }
    }
  edoActLed = digitalRead(ledPin);
  
}

void loop() {
  if (interruptCounter > 0) {           // LED de actividad parpadea
    edoAntLed = edoActLed;
    edoActLed = digitalRead(ledPin);
    edoLed = !edoLed;
    digitalWrite(ledPin, edoLed);
  }
  // Inicia la detección de la ráfaga IR que viene sel sensor VR38
  // Un bit 1 dura T11 (589 us). Un bit 0 dura T17 ( 1723 us) ambos en LOW
  
  if (irrecv.decode(&results)) {
    digitalWrite(uno, HIGH);
    
    if (results.value){   // >> 16){
      digitalWrite(dos, HIGH);
      Serial.print((uint32_t) (results.value >> 32), HEX);
      comando = results.value & 0xFFFFFFFF;
      Serial.println((uint32_t) (comando & 0xFFFFFFFF), HEX);
      SerialBT.write(comando);
    }
    
    delay(500);
    irrecv.resume();    // Listo para recibir el siguiente valor
    digitalWrite(uno, LOW);
    digitalWrite(dos, LOW);
    Serial.print("El valor del codigo es: ");
    Serial.println(results.value, HEX);
    Serial.print("El ADD del codigo es: ");
    Serial.println(results.address);
    Serial.print("El CMD del codigo es: ");
    Serial.println(results.command);
    Serial.println("\n");
    bandera = 1;
  }
  
 delay(500);
}
